@@ -17,22 +17,25 @@
 package org.apache.commons.configuration;
 
 import java.io.File;
+import java.io.IOException;
 import java.util.Collection;
 import java.util.Set;
+import java.util.Map;
+import java.util.HashMap;
+
+import junit.framework.TestCase;
 
 import org.apache.commons.configuration.beanutils.BeanHelper;
 import org.apache.commons.configuration.reloading.FileChangedReloadingStrategy;
 import org.apache.commons.configuration.tree.DefaultConfigurationNode;
 import org.apache.commons.configuration.tree.xpath.XPathExpressionEngine;
-
-import junit.framework.TestCase;
+import org.apache.commons.lang.text.StrLookup;
 
 /**
  * Test class for DefaultConfigurationBuilder.
  *
  * @author Oliver Heger
- * @version $Id: TestDefaultConfigurationBuilder.java 384601 2006-03-09
- * 20:22:58Z oheger $
+ * @version $Id: TestDefaultConfigurationBuilder.java 727834 2008-12-18 22:16:32Z rgoers $
  */
 public class TestDefaultConfigurationBuilder extends TestCase
 {
@@ -55,6 +58,24 @@ public class TestDefaultConfigurationBuilder extends TestCase
     private static final File INIT_FILE = new File(
             "conf/testComplexInitialization.xml");
 
+    private static final File CLASS_FILE = new File(
+            "conf/testExtendedClass.xml");
+
+    private static final File PROVIDER_FILE = new File(
+            "conf/testConfigurationProvider.xml");
+
+    private static final File EXTENDED_PROVIDER_FILE = new File(
+            "conf/testExtendedXMLConfigurationProvider.xml");
+
+    private static final File GLOBAL_LOOKUP_FILE = new File(
+            "conf/testGlobalLookup.xml");
+
+    private static final File SYSTEM_PROPS_FILE = new File(
+            "conf/testSystemProperties.xml");
+
+    /** Constant for the name of an optional configuration.*/
+    private static final String OPTIONAL_NAME = "optionalConfig";
+
     /** Stores the object to be tested. */
     DefaultConfigurationBuilder factory;
 
@@ -63,10 +84,11 @@ public class TestDefaultConfigurationBuilder extends TestCase
         super.setUp();
         System
                 .setProperty("java.naming.factory.initial",
-                        "org.apache.commons.configuration.MockStaticMemoryInitialContextFactory");
+                        "org.apache.commons.configuration.MockInitialContextFactory");
         System.setProperty("test_file_xml", "test.xml");
         System.setProperty("test_file_combine", "testcombine1.xml");
         factory = new DefaultConfigurationBuilder();
+        factory.clearErrorListeners();  // avoid exception messages
     }
 
     /**
@@ -144,25 +166,25 @@ public class TestDefaultConfigurationBuilder extends TestCase
      */
     public void testConfigurationDeclarationGetAttributes()
     {
-        factory.addProperty("/ xml/fileName", "test.xml");
+        factory.addProperty("xml.fileName", "test.xml");
         DefaultConfigurationBuilder.ConfigurationDeclaration decl = new DefaultConfigurationBuilder.ConfigurationDeclaration(
                 factory, factory.configurationAt("xml"));
         assertNull("Found an at attribute", decl.getAt());
         assertFalse("Found an optional attribute", decl.isOptional());
-        factory.addProperty("/xml @config-at", "test1");
+        factory.addProperty("xml[@config-at]", "test1");
         assertEquals("Wrong value of at attribute", "test1", decl.getAt());
-        factory.addProperty("/xml @at", "test2");
+        factory.addProperty("xml[@at]", "test2");
         assertEquals("Wrong value of config-at attribute", "test1", decl.getAt());
-        factory.clearProperty("/xml/@config-at");
+        factory.clearProperty("xml[@config-at]");
         assertEquals("Old at attribute not detected", "test2", decl.getAt());
-        factory.addProperty("/xml @config-optional", "true");
+        factory.addProperty("xml[@config-optional]", "true");
         assertTrue("Wrong value of optional attribute", decl.isOptional());
-        factory.addProperty("/xml @optional", "false");
+        factory.addProperty("xml[@optional]", "false");
         assertTrue("Wrong value of config-optional attribute", decl.isOptional());
-        factory.clearProperty("/xml/@config-optional");
-        factory.setProperty("/xml/@optional", Boolean.TRUE);
+        factory.clearProperty("xml[@config-optional]");
+        factory.setProperty("xml[@optional]", Boolean.TRUE);
         assertTrue("Old optional attribute not detected", decl.isOptional());
-        factory.setProperty("/xml/@optional", "invalid value");
+        factory.setProperty("xml[@optional]", "invalid value");
         try
         {
             decl.isOptional();
@@ -246,7 +268,7 @@ public class TestDefaultConfigurationBuilder extends TestCase
         factory.addConfigurationProvider("test",
                 new DefaultConfigurationBuilder.ConfigurationProvider(
                         PropertiesConfiguration.class));
-        factory.addProperty("/ test@throwExceptionOnMissing", "true");
+        factory.addProperty("test[@throwExceptionOnMissing]", "true");
         DefaultConfigurationBuilder.ConfigurationDeclaration decl = new DefaultConfigurationBuilder.ConfigurationDeclaration(
                 factory, factory.configurationAt("test"));
         PropertiesConfiguration conf = (PropertiesConfiguration) BeanHelper
@@ -261,7 +283,7 @@ public class TestDefaultConfigurationBuilder extends TestCase
      */
     public void testConfigurationBeanFactoryCreateUnknownTag()
     {
-        factory.addProperty("/ test@throwExceptionOnMissing", "true");
+        factory.addProperty("test[@throwExceptionOnMissing]", "true");
         DefaultConfigurationBuilder.ConfigurationDeclaration decl = new DefaultConfigurationBuilder.ConfigurationDeclaration(
                 factory, factory.configurationAt("test"));
         try
@@ -397,6 +419,16 @@ public class TestDefaultConfigurationBuilder extends TestCase
                 .getString("test.configuration"));
     }
 
+    /**
+     * Tests whether a default log error listener is registered at the builder
+     * instance.
+     */
+    public void testLogErrorListener()
+    {
+        assertEquals("No default error listener registered", 1,
+                new DefaultConfigurationBuilder().getErrorListeners().size());
+    }
+
     /**
      * Tests loading a definition file that contains optional configurations.
      */
@@ -408,6 +440,20 @@ public class TestDefaultConfigurationBuilder extends TestCase
         assertEquals("value", config.getProperty("element"));
     }
 
+    /**
+     * Tests whether loading a failing optional configuration causes an error
+     * event.
+     */
+    public void testLoadOptionalErrorEvent() throws Exception
+    {
+        factory.clearErrorListeners();
+        ConfigurationErrorListenerImpl listener = new ConfigurationErrorListenerImpl();
+        factory.addErrorListener(listener);
+        prepareOptionalTest("configuration", false);
+        listener.verify(DefaultConfigurationBuilder.EVENT_ERR_LOAD_OPTIONAL,
+                OPTIONAL_NAME, null);
+    }
+
     /**
      * Tests loading a definition file with optional and non optional
      * configuration sources. One non optional does not exist, so this should
@@ -434,18 +480,93 @@ public class TestDefaultConfigurationBuilder extends TestCase
      */
     public void testLoadOptionalNonFileBased() throws ConfigurationException
     {
-        factory.addProperty("/ override/configuration@fileName",
-                "nonExisting.xml");
-        factory.addProperty("/override/configuration[1] @config-optional",
-                Boolean.TRUE);
-        factory.addProperty("/override/configuration[1] @config-name",
-                "optionalConfig");
-        CombinedConfiguration config = factory.getConfiguration(false);
+        CombinedConfiguration config = prepareOptionalTest("configuration", false);
         assertTrue("Configuration not empty", config.isEmpty());
         assertEquals("Wrong number of configurations", 0, config
                 .getNumberOfConfigurations());
     }
 
+    /**
+     * Tests an optional, non existing configuration with the forceCreate
+     * attribute. This configuration should be added to the resulting
+     * configuration.
+     */
+    public void testLoadOptionalForceCreate() throws ConfigurationException
+    {
+        factory.setBasePath(TEST_FILE.getParent());
+        CombinedConfiguration config = prepareOptionalTest("xml", true);
+        assertEquals("Wrong number of configurations", 1, config
+                .getNumberOfConfigurations());
+        FileConfiguration fc = (FileConfiguration) config
+                .getConfiguration(OPTIONAL_NAME);
+        assertNotNull("Optional config not found", fc);
+        assertEquals("File name was not set", "nonExisting.xml", fc
+                .getFileName());
+        assertNotNull("Base path was not set", fc.getBasePath());
+    }
+
+    /**
+     * Tests loading an embedded optional configuration builder with the force
+     * create attribute.
+     */
+    public void testLoadOptionalBuilderForceCreate()
+            throws ConfigurationException
+    {
+        CombinedConfiguration config = prepareOptionalTest("configuration",
+                true);
+        assertEquals("Wrong number of configurations", 1, config
+                .getNumberOfConfigurations());
+        assertTrue(
+                "Wrong optional configuration type",
+                config.getConfiguration(OPTIONAL_NAME) instanceof CombinedConfiguration);
+    }
+
+    /**
+     * Tests loading an optional configuration with the force create attribute
+     * set. The provider will always throw an exception. In this case the
+     * configuration will not be added to the resulting combined configuration.
+     */
+    public void testLoadOptionalForceCreateWithException()
+            throws ConfigurationException
+    {
+        factory.addConfigurationProvider("test",
+                new DefaultConfigurationBuilder.ConfigurationBuilderProvider()
+                {
+                    // Throw an exception here, too
+                    public AbstractConfiguration getEmptyConfiguration(
+                            DefaultConfigurationBuilder.ConfigurationDeclaration decl) throws Exception
+                    {
+                        throw new Exception("Unable to create configuration!");
+                    }
+                });
+        CombinedConfiguration config = prepareOptionalTest("test", true);
+        assertEquals("Optional configuration could be created", 0, config
+                .getNumberOfConfigurations());
+    }
+
+    /**
+     * Prepares a test for loading a configuration definition file with an
+     * optional configuration declaration.
+     *
+     * @param tag the tag name with the optional configuration
+     * @param force the forceCreate attribute
+     * @return the combined configuration obtained from the builder
+     * @throws ConfigurationException if an error occurs
+     */
+    private CombinedConfiguration prepareOptionalTest(String tag, boolean force)
+            throws ConfigurationException
+    {
+        String prefix = "override." + tag;
+        factory.addProperty(prefix + "[@fileName]", "nonExisting.xml");
+        factory.addProperty(prefix + "[@config-optional]", Boolean.TRUE);
+        factory.addProperty(prefix + "[@config-name]", OPTIONAL_NAME);
+        if (force)
+        {
+            factory.addProperty(prefix + "[@config-forceCreate]", Boolean.TRUE);
+        }
+        return factory.getConfiguration(false);
+    }
+
     /**
      * Tests loading a definition file with multiple different sources.
      */
@@ -485,7 +606,7 @@ public class TestDefaultConfigurationBuilder extends TestCase
      */
     public void testSetConfigurationBasePath() throws ConfigurationException
     {
-        factory.addProperty("/ properties@fileName", "test.properties");
+        factory.addProperty("properties[@fileName]", "test.properties");
         File deepDir = new File("conf/config/deep");
         factory.setConfigurationBasePath(deepDir.getAbsolutePath());
 
@@ -593,11 +714,164 @@ public class TestDefaultConfigurationBuilder extends TestCase
     public void testConfigurationBuilderProvider()
             throws ConfigurationException
     {
-        factory.addProperty("/ override/configuration@fileName", TEST_FILE
+        factory.addProperty("override.configuration[@fileName]", TEST_FILE
                 .getAbsolutePath());
         CombinedConfiguration cc = factory.getConfiguration(false);
         assertEquals("Wrong number of configurations", 1, cc
                 .getNumberOfConfigurations());
         checkProperties(cc);
     }
+
+    /**
+     * Tests whether XML settings can be inherited.
+     */
+    public void testLoadXMLWithSettings() throws ConfigurationException,
+            IOException
+    {
+        File confDir = new File("conf");
+        File targetDir = new File("target");
+        File testXMLSource = new File(confDir, "testDtd.xml");
+        File testXMLValidationSource = new File(confDir,
+                "testValidateInvalid.xml");
+        File testSavedXML = new File(targetDir, "testSave.xml");
+        File testSavedFactory = new File(targetDir, "testSaveFactory.xml");
+        File dtdFile = new File(confDir, "properties.dtd");
+        final String publicId = "http://commons.apache.org/test.dtd";
+
+        XMLConfiguration config = new XMLConfiguration(testXMLSource);
+        config.setPublicID(publicId);
+        config.save(testSavedXML);
+        factory.addProperty("xml[@fileName]", testSavedXML.getAbsolutePath());
+        factory.addProperty("xml(0)[@validating]", "true");
+        factory.addProperty("xml(-1)[@fileName]", testXMLValidationSource
+                .getAbsolutePath());
+        factory.addProperty("xml(1)[@config-optional]", "true");
+        factory.addProperty("xml(1)[@validating]", "true");
+        factory.save(testSavedFactory);
+
+        factory = new DefaultConfigurationBuilder();
+        factory.setFile(testSavedFactory);
+        factory.registerEntityId(publicId, dtdFile.toURL());
+        factory.clearErrorListeners();
+        Configuration c = factory.getConfiguration();
+        assertEquals("Wrong property value", "value1", c.getString("entry(0)"));
+        assertFalse("Invalid XML source was loaded", c
+                .containsKey("table.name"));
+
+        testSavedXML.delete();
+        testSavedFactory.delete();
+    }
+
+    /**
+     * Tests loading a configuration definition file that defines a custom
+     * result class.
+     */
+    public void testExtendedClass() throws ConfigurationException
+    {
+        factory.setFile(CLASS_FILE);
+        CombinedConfiguration cc = factory.getConfiguration(true);
+        assertEquals("Extended", cc.getProperty("test"));
+        assertTrue("Wrong result class: " + cc.getClass(),
+                cc instanceof ExtendedCombinedConfiguration);
+    }
+
+    /**
+     * Tests loading a configuration definition file that defines new providers.
+     */
+    public void testConfigurationProvider() throws ConfigurationException
+    {
+        factory.setFile(PROVIDER_FILE);
+        factory.getConfiguration(true);
+        DefaultConfigurationBuilder.ConfigurationProvider provider = factory
+                .providerForTag("test");
+        assertNotNull("Provider 'test' not registered", provider);
+    }
+
+        /**
+     * Tests loading a configuration definition file that defines new providers.
+     */
+    public void testExtendedXMLConfigurationProvider() throws ConfigurationException
+    {
+        factory.setFile(EXTENDED_PROVIDER_FILE);
+        CombinedConfiguration cc = factory.getConfiguration(true);
+        DefaultConfigurationBuilder.ConfigurationProvider provider = factory
+                .providerForTag("test");
+        assertNotNull("Provider 'test' not registered", provider);
+        Configuration config = cc.getConfiguration("xml");
+        assertNotNull("Test configuration not present", config);
+        assertTrue("Configuration is not ExtendedXMLConfiguration, is " +
+                config.getClass().getName(), config instanceof ExtendedXMLConfiguration);
+    }
+
+    public void testGlobalLookup() throws Exception
+    {
+        factory.setFile(GLOBAL_LOOKUP_FILE);
+        CombinedConfiguration cc = factory.getConfiguration(true);
+        String value = cc.getInterpolator().lookup("test:test_key");
+        assertNotNull("The test key was not located", value);
+        assertEquals("Incorrect value retrieved","test.value",value);       
+    }
+         
+    public void testSystemProperties() throws Exception
+    {
+        factory.setFile(SYSTEM_PROPS_FILE);
+        CombinedConfiguration cc = factory.getConfiguration(true);
+        String value = System.getProperty("key1");
+        assertNotNull("The test key was not located", value);
+        assertEquals("Incorrect value retrieved","value1",value);
+    }
+
+
+    /**
+     * A specialized combined configuration implementation used for testing
+     * custom result classes.
+     */
+    public static class ExtendedCombinedConfiguration extends
+            CombinedConfiguration
+    {
+        /**
+         * The serial version UID.
+         */
+        private static final long serialVersionUID = 4678031745085083392L;
+
+        public Object getProperty(String key)
+        {
+            if (key.equals("test"))
+            {
+                return "Extended";
+            }
+            return super.getProperty(key);
+        }
+    }
+
+    public static class ExtendedXMLConfiguration extends XMLConfiguration
+    {
+        public ExtendedXMLConfiguration()
+        {
+        }
+
+    }
+
+    public static class TestLookup extends StrLookup
+    {
+        Map map = new HashMap();
+
+        public TestLookup()
+        {
+            map.put("test_file_xml", "test.xml");
+            map.put("test_file_combine", "testcombine1.xml");
+            map.put("test_key", "test.value");
+        }
+
+        public String lookup(String key)
+        {
+            if (key == null)
+            {
+                return null;
+            }
+            return (String)map.get(key);
+
+        }
+    }
 }
+
